// $Id: TYYYParameter.cxx 854 2012-06-06 14:59:49Z adamczew $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------

#include "TYYYParameter.h"

#include "go4iostream.h"

#include "TGo4Fitter.h"

TYYYParameter::TYYYParameter() :
   TGo4Parameter("YYYParameter")
{
   frP1=0;
   frP2=0;

   for(Int_t ix=0;ix<__ARRAYSIZE__;ix++) {
      fiDataArray[ix]=0;
      fxFitArray[ix]=0;
   }
}

TYYYParameter::TYYYParameter(const char* name) :
   TGo4Parameter(name)
{
   frP1 = 10;
   frP2 = 20;
   for(Int_t ix=0;ix<__ARRAYSIZE__;ix++) {
      fiDataArray[ix]=ix+30;
      fxFitArray[ix] = new TGo4Fitter();
   }
}

TYYYParameter::~TYYYParameter()
{
   for(Int_t ix=0;ix<__ARRAYSIZE__;++ix)
     if(fxFitArray[ix]) {
        delete fxFitArray[ix];
        fxFitArray[ix] = 0;
     }
}

Int_t TYYYParameter::PrintParameter(Text_t * n, Int_t)
{
  cout << "Parameter " << GetName()<<":" <<endl;
  cout << " P1="<<frP1<<endl;
  cout << " P2="<<frP2<< endl;
  for(Int_t ix=0;ix<__ARRAYSIZE__;++ix) {
     cout << "fiDataArray["<<ix<<"]="<<fiDataArray[ix]<<endl;
     if (fxFitArray[ix]) fxFitArray[ix]->PrintLines();
  }
  return 0;
}

Bool_t TYYYParameter::UpdateFrom(TGo4Parameter *source)
{
   TYYYParameter* from = dynamic_cast<TYYYParameter*> (source);
   if (from==0) {
      cout << "Wrong parameter class: " << source->ClassName() << endl;
      return kFALSE;
   }

   cout << "Parameter " << GetName() << ": P1=" << frP1 << " P2="<< frP2 << endl;
   frP1 = from->frP1;
   frP2 = from->frP2;
   for(Int_t ix=0;ix<__ARRAYSIZE__;++ix) {
      fiDataArray[ix] = from->fiDataArray[ix];
      // replace old fitters by copy of source fitter:
      if(fxFitArray[ix]) {
         delete fxFitArray[ix];
         fxFitArray[ix] = 0;
      }
      if (from->fxFitArray[ix])
         fxFitArray[ix] = (TGo4Fitter*) from->fxFitArray[ix]->Clone();
   }
   return kTRUE;
}
